package dev.svitan.routes

import dev.svitan.plugins.AuthorizationException
import dev.svitan.schemas.ActionService
import dev.svitan.schemas.AuthService
import dev.svitan.schemas.NewActionDTO
import dev.svitan.schemas.RunActionDTO
import dev.svitan.schemas.sha256
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.plugins.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import java.util.*

fun Application.routeAction() {
    routing {
        authentication {
            get("/action") {
                call.respond(ActionService.readAll())
            }

            post("/action") {
                val action = call.receive<NewActionDTO>()
                call.respond(HttpStatusCode.Created, ActionService.create(action))
            }

            get("/action/{id}") {
                val idRaw = call.parameters["id"] ?: throw BadRequestException("Invalid id")
                val id = UUID.fromString(idRaw)
                val action = ActionService.read(id) ?: throw NotFoundException()
                call.respond(action)
            }

            post("/action/{id}") {
                val idRaw = call.parameters["id"] ?: throw BadRequestException("Invalid id")
                val id = UUID.fromString(idRaw)
            }

            put("/action/{id}") {
                val idRaw = call.parameters["id"] ?: throw BadRequestException("Invalid id")
                val id = UUID.fromString(idRaw)

                val action = ActionService.read(id) ?: throw NotFoundException()
                val run = call.receive<RunActionDTO>()
                val auth = AuthService.validate(
                    UUID.fromString(action.authId),
                    run.pin,
                    run.key
                ) ?: throw AuthorizationException("Invalid pin or key")

                // TODO: run the action

                call.respond(HttpStatusCode.NoContent)
            }

            delete("/action/{id}") {
                val idRaw = call.parameters["id"] ?: throw BadRequestException("Invalid id")
                val id = UUID.fromString(idRaw)
                ActionService.delete(id)
                call.respond(HttpStatusCode.NoContent)
            }
        }
    }
}
